#
# CIP 2011 vehicle routing problem.
#
# Ralf Borndoerfer
# 11/2011
#
# Input file "trips.dat" contains trips that have to be done, first trip is
# from the depot to the depot. File format is "1s 2n 3s 4n 5s", where
#
# 1s = trip number                (0..n)
# 2n = y-coordinate start of trip (1..7)
# 3s = x-coordinate start of trip (1..7)
# 4n = y-coordinate end   of trip (A..F)
# 5s = x-coordinate end   of trip (A..F)

# identify
do print "CIP 2011 vehicle routing problem";

# params
param DAT := "trips.dat";
param FIX := 10; # cost for a vehicle
param LEN := 30; # max route length

# print params
do print "FIX = ", FIX;
do print "LEN = ", LEN;

# nodes, depot (first node), trips, arcs
set V := { read DAT as "<1n>" comment "#" };
param depot := ord(V,1,1);
set W := V \ { depot };

# trip coordinates from {1..7} * {A..F} * {1..7} * {A..F}
param y1[V] := read DAT as "<1n> 2n" comment "#";
param x1[V] := read DAT as "<1n> 3s" comment "#";
param y2[V] := read DAT as "<1n> 4n" comment "#";
param x2[V] := read DAT as "<1n> 5s" comment "#";

# character coordinates
set CHAR := { "A", "B", "C", "D", "E", "F" };
param ind[CHAR] := <"A"> 1, <"B"> 2, <"C"> 3, <"D"> 4, <"E"> 5, <"F"> 6;
do forall <v> in V do check <x1[v]> in CHAR;
do forall <v> in V do check <x2[v]> in CHAR;

# distance from the start of trip u to the end of trip u to the start of trip v
defnumb dist(u,v) := 2 * abs(ind[x2[u]]-ind[x1[u]]) + abs(y2[u]-y1[u])
				   + 2 * abs(ind[x2[u]]-ind[x1[v]]) + abs(y2[u]-y1[v]);
do check dist(depot,depot) == 0;

# your turn ...

